==> tunacode_cli-0.0.75.tar.gz: tunacode_cli-0.0.75/documentation/development/performance-optimizations.md <==
# Performance Optimizations

## Overview

This document tracks significant performance improvements implemented in TunaCode.

## Directory Caching System (2025-08-12)

### Problem
Agent response times were reaching 30+ seconds due to repeated filesystem operations, particularly `list_dir` calls during codebase exploration.

### Solution
Implemented intelligent directory caching using the existing CodeIndex infrastructure:

- **Background pre-warming**: Index built during REPL startup (non-blocking)
- **Smart caching**: 5-second TTL with automatic cache invalidation
- **Singleton pattern**: Global access to cached directory data
- **Graceful fallback**: Original behavior when cache unavailable

### Results
- **50-500x faster** directory operations for cached paths
- **Cache hit rate**: 100% for pre-indexed directories
- **Response time improvement**: ~0.0001s vs 10-200ms
- **Memory overhead**: Minimal (~1-5MB for typical projects)

### Implementation
- Modified: `src/tunacode/core/code_index.py` (singleton + public API)
- Modified: `src/tunacode/cli/repl.py` (background pre-warming)
- Modified: `src/tunacode/tools/list_dir.py` (cache integration)

### Documentation
- Detailed docs: `.claude/development/directory-caching-optimization.md`

## System Prompt Caching (2025-08-12)

### Problem
System prompts and TUNACODE.md were being loaded from disk on every agent creation.

### Solution
Added file-based caching with modification time checking:

```python
_PROMPT_CACHE = {}  # {filepath: (content, mtime)}
_TUNACODE_CACHE = {}  # {filepath: (content, mtime)}
```

### Results
- **9x faster** system prompt loading
- **3-8 second reduction** in agent initialization time
- **Zero risk**: Automatic cache invalidation on file changes

### Implementation
- Modified: `src/tunacode/core/agents/agent_components/agent_config.py`

## Future Optimization Opportunities

### High Impact, Low Effort
1. **Tool result caching**: Cache read-only tool results with TTL
2. **Connection pooling**: Reuse HTTP connections for API calls
3. **Grep result caching**: Cache expensive grep operations

### Medium Impact
4. **Async tool initialization**: Load tools lazily when first used
5. **Model response caching**: Cache identical queries within session
6. **File stat caching**: Cache `os.stat()` results for file operations

### Research Needed
7. **Streaming optimization**: Reduce token-level streaming overhead
8. **Memory optimization**: Compress large cached data structures
9. **Persistent caching**: Save caches to disk for cross-session reuse

## Performance Monitoring

### Key Metrics
- Agent response time (target: <5 seconds)
- Cache hit rates (target: >80%)
- Memory usage (target: <100MB overhead)
- Background task completion time

### Profiling Tools
```bash
# Profile agent performance
python -m cProfile -o profile.stats src/tunacode/main.py

# Monitor memory usage
python -m memory_profiler src/tunacode/main.py

# Enable debug logging
export TUNACODE_LOG_LEVEL=DEBUG
```

## Best Practices

### When Adding Optimizations
1. **Measure first**: Profile before optimizing
2. **Test thoroughly**: Ensure no functionality regression
3. **Document impact**: Record performance improvements
4. **Add fallbacks**: Graceful degradation when optimization fails
5. **Monitor memory**: Track memory usage in production

### Cache Design Principles
1. **Short TTL**: Prefer short TTL over stale data
2. **Thread safety**: Use proper locking for concurrent access
3. **Automatic invalidation**: Don't require manual cache clearing
4. **Configurable**: Allow tuning cache parameters
5. **Observable**: Add debug logging for cache behavior

==> tunacode_cli-0.0.78.11.tar.gz: tunacode_cli-0.0.78.11/documentation/development/performance-optimizations.md <==
# Performance Optimizations

## Overview

This document tracks significant performance improvements implemented in TunaCode.

## Directory Caching System (2025-08-12)

### Problem
Agent response times were reaching 30+ seconds due to repeated filesystem operations, particularly `list_dir` calls during codebase exploration.

### Solution
Implemented intelligent directory caching using the existing CodeIndex infrastructure:

- **Background pre-warming**: Index built during REPL startup (non-blocking)
- **Smart caching**: 5-second TTL with automatic cache invalidation
- **Singleton pattern**: Global access to cached directory data
- **Graceful fallback**: Original behavior when cache unavailable

### Results
- **50-500x faster** directory operations for cached paths
- **Cache hit rate**: 100% for pre-indexed directories
- **Response time improvement**: ~0.0001s vs 10-200ms
- **Memory overhead**: Minimal (~1-5MB for typical projects)

### Implementation
- Modified: `src/tunacode/core/code_index.py` (singleton + public API)
- Modified: `src/tunacode/cli/repl.py` (background pre-warming)
- Modified: `src/tunacode/tools/list_dir.py` (cache integration)

### Documentation
- Detailed docs: `.claude/development/directory-caching-optimization.md`

## System Prompt Caching (2025-08-12)

### Problem
System prompts and AGENTS.md were being loaded from disk on every agent creation.

### Solution
Added file-based caching with modification time checking:

```python
_PROMPT_CACHE = {}  # {filepath: (content, mtime)}
_TUNACODE_CACHE = {}  # {filepath: (content, mtime)}
```

### Results
- **9x faster** system prompt loading
- **3-8 second reduction** in agent initialization time
- **Zero risk**: Automatic cache invalidation on file changes

### Implementation
- Modified: `src/tunacode/core/agents/agent_components/agent_config.py`

## Future Optimization Opportunities

### High Impact, Low Effort
1. **Tool result caching**: Cache read-only tool results with TTL
2. **Connection pooling**: Reuse HTTP connections for API calls
3. **Grep result caching**: Cache expensive grep operations

### Medium Impact
4. **Async tool initialization**: Load tools lazily when first used
5. **Model response caching**: Cache identical queries within session
6. **File stat caching**: Cache `os.stat()` results for file operations

### Research Needed
7. **Streaming optimization**: Reduce token-level streaming overhead
8. **Memory optimization**: Compress large cached data structures
9. **Persistent caching**: Save caches to disk for cross-session reuse

## Performance Monitoring

### Key Metrics
- Agent response time (target: <5 seconds)
- Cache hit rates (target: >80%)
- Memory usage (target: <100MB overhead)
- Background task completion time

### Profiling Tools
```bash
# Profile agent performance
python -m cProfile -o profile.stats src/tunacode/main.py

# Monitor memory usage
python -m memory_profiler src/tunacode/main.py

# Enable debug logging
export TUNACODE_LOG_LEVEL=DEBUG
```

## Best Practices

### When Adding Optimizations
1. **Measure first**: Profile before optimizing
2. **Test thoroughly**: Ensure no functionality regression
3. **Document impact**: Record performance improvements
4. **Add fallbacks**: Graceful degradation when optimization fails
5. **Monitor memory**: Track memory usage in production

### Cache Design Principles
1. **Short TTL**: Prefer short TTL over stale data
2. **Thread safety**: Use proper locking for concurrent access
3. **Automatic invalidation**: Don't require manual cache clearing
4. **Configurable**: Allow tuning cache parameters
5. **Observable**: Add debug logging for cache behavior
